/*
 *  Analóg multiplexer minta kód
 *  ESP8266 / NodeMCU -ra
 *  techfactory.hu
 */
 
//az analog multiplexerünknek 3 digitális pin kell
//amivel "kapcsolgatni" tudjuk, hogy épp melyik analóg bemenet
//értékére vagyunk kiváncsiak
#define AMS0 D1
#define AMS1 D2
#define AMS2 D3

//analog bemenetek száma nodeMCU-ra (ESP8266)
//8-al egészítjük ki a CD4051 segítségével
//ESP8266 esetén (ha nem nodeMCU boardon van) akkor az analóg bemenetére feszültségosztót kell kötni, 
//ugyanis 0 és 1 V között mér
//nem 0 és 3.3V között!
#define ANALOG_INPUTS 8

float temp = 0;
float hum = 0;

void setup() {
   Serial.begin(115200);
   Serial.println("Starting up");
   pinMode(AMS0, OUTPUT);
   pinMode(AMS1, OUTPUT);
   pinMode(AMS2, OUTPUT);
}

void loop() {
   int value = 0;
   
   //olvassuk le az analóg értékeket
   for (int i=0; i<ANALOG_INPUTS; i++) {
      Serial.print("A"+String(i)+" = ");
      value = analogReadCD4051(i); 
      Serial.println(value);
      
      //homerseklet
      if (i==0) {
         temp = map_float(value, 0, 1024, -20, 80);
      //paratartalom
      }else if(i==1) {
         hum = map_float(value, 0, 1024, 0, 100);
      }
   }
   
   Serial.println("H = "+String(temp)+" C");
   Serial.println("P = "+String(hum)+" %");

   //5mp késlelteltés
   delay(5000);
}

int analogReadCD4051(uint8_t pin) {
    //a digitális kimenetek segítségével átváltunk egy analóg bemenetre
    //így annak értékét továbbítja a multiplexer az A0-ra
    
  /*  if (pin == 0) {
  		digitalWrite(AMS0, 0);
  		digitalWrite(AMS1, 0);
  		digitalWrite(AMS2, 0);
    }else if(pin == 1) {	
  		digitalWrite(AMS0, 1);
  		digitalWrite(AMS1, 0);
  		digitalWrite(AMS2, 0);
    }else if(pin == 2) {	
  		digitalWrite(AMS0, 0);
  		digitalWrite(AMS1, 1);
  		digitalWrite(AMS2, 0);
    }else if(pin == 3) {	
  		digitalWrite(AMS0, 1);
  		digitalWrite(AMS1, 1);
  		digitalWrite(AMS2, 0);
    }else if(pin == 4) {	
  		digitalWrite(AMS0, 0);
  		digitalWrite(AMS1, 0);
  		digitalWrite(AMS2, 1);
    }else if(pin == 5) {	
  		digitalWrite(AMS0, 1);
  		digitalWrite(AMS1, 0);
  		digitalWrite(AMS2, 1);
    }else if(pin == 6) {	
  		digitalWrite(AMS0, 0);
  		digitalWrite(AMS1, 1);
  		digitalWrite(AMS2, 1);
    }else if(pin == 7) {	
  		digitalWrite(AMS0, 1);
  		digitalWrite(AMS1, 1);
  		digitalWrite(AMS2, 1);
    }*/
    
    //A sok if else elágazás helyett biteltolással egy
    //elegáns megoldással orvosolható a digitális pinek értékének megfelelő változtatása
    //https://github.com/ajfisher/arduino-analog-multiplexer/blob/master/analogmuxdemux.cpp
    //Bitműveletek:
    //https://hu.wikipedia.org/wiki/Bitművelet
    
    digitalWrite(AMS0, (pin & 0x01));
    digitalWrite(AMS1, ((pin>>1) & 0x01));
    digitalWrite(AMS2, ((pin>>2) & 0x01));
    
    //leolvassuk a node-mcu analóg bemenetét, hiszen
    //a multiplexer a beállított analóg pin értékét ide továbbítja
    return analogRead(A0); 
}

float map_float(long x, long in_min, long in_max, long out_min, long out_max) {
    return (float)(x - in_min) * (out_max - out_min) / (float)(in_max - in_min) + out_min;
}
